@@ -65,6 +65,7 @@ struct _GeditDialogReplace {
 	GtkWidget *match_case_checkbutton;
 	GtkWidget *entire_word_checkbutton;
 	GtkWidget *wrap_around_checkbutton;
+	GtkWidget *search_backwards_checkbutton;
 };
 
 struct _GeditDialogFind {
@@ -76,6 +77,7 @@ struct _GeditDialogFind {
 	GtkWidget *match_case_checkbutton;
 	GtkWidget *entire_word_checkbutton;
 	GtkWidget *wrap_around_checkbutton;
+	GtkWidget *search_backwards_checkbutton;
 };
 
 static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
@@ -96,6 +98,7 @@ static GeditDialogFind    *dialog_find_get_dialog 	(void);
 static GQuark was_wrap_around_id = 0;
 static GQuark was_entire_word_id = 0;
 static GQuark was_case_sensitive_id = 0;
+static GQuark was_search_backwards_id = 0;
 
 GQuark 
 gedit_was_wrap_around_quark (void)
@@ -135,6 +138,7 @@ text_not_found_dialog (const gchar *text, GtkWindow *parent)
 	gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
 
 	gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (message_dlg), FALSE);
 	
 	gtk_dialog_run (GTK_DIALOG (message_dlg));
   	gtk_widget_destroy (message_dlg);
@@ -235,15 +239,17 @@ dialog_replace_get_dialog (void)
 	GladeXML *gui;
 	GtkWindow *window;
 	GtkWidget *content;
-	GtkWidget *button;
 	GtkWidget *replace_with_label;
 	
 	gedit_debug (DEBUG_SEARCH, "");
 	
+	window = GTK_WINDOW (gedit_get_active_window ());
+
 	if (dialog != NULL)
 	{
-		gdk_window_show (dialog->dialog->window);
-		gdk_window_raise (dialog->dialog->window);
+		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
+					      GTK_WINDOW (window));
+		gtk_window_present (GTK_WINDOW (dialog->dialog));
 		gtk_widget_grab_focus (dialog->dialog);
 
 		return dialog;
@@ -251,16 +257,14 @@ dialog_replace_get_dialog (void)
 
 	gui = glade_xml_new ( GEDIT_GLADEDIR "replace.glade2",
 			     "replace_dialog_content", NULL);
-
-	if (!gui) {
-		g_warning
-		    ("Could not find replace.glade2, reinstall gedit.\n");
+	if (!gui)
+	{
+		gedit_warning (window,
+			       MISSING_FILE,
+			       GEDIT_GLADEDIR "replace.glade2");
 		return NULL;
 	}
 
-	window = GTK_WINDOW (bonobo_mdi_get_active_window
-			     (BONOBO_MDI (gedit_mdi)));
-
 	dialog = g_new0 (GeditDialogReplace, 1);
 
 	dialog->dialog = gtk_dialog_new_with_buttons (_("Replace"),
@@ -272,22 +276,16 @@ dialog_replace_get_dialog (void)
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
-	/* Add Replace All button */
+	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->dialog), FALSE);
+
 	gtk_dialog_add_button (GTK_DIALOG (dialog->dialog), 
 			       _("Replace _All"), GEDIT_RESPONSE_REPLACE_ALL);
-	
-	/* Add Replace button */
-	button = gedit_button_new_with_stock_image (_("_Replace"), GTK_STOCK_FIND_AND_REPLACE);
-	g_return_val_if_fail (button != NULL, NULL);
 
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gedit_dialog_add_button (GTK_DIALOG (dialog->dialog),
+				 _("_Replace"), GTK_STOCK_FIND_AND_REPLACE,
+				 GEDIT_RESPONSE_REPLACE);
 
-	gtk_widget_show (button);
-
-	gtk_dialog_add_action_widget (GTK_DIALOG (dialog->dialog), 
-				      button, GEDIT_RESPONSE_REPLACE);
-
-	/* Add Find button */
 	gtk_dialog_add_button (GTK_DIALOG (dialog->dialog), 
 			       GTK_STOCK_FIND, GEDIT_RESPONSE_FIND);
 
@@ -300,6 +298,7 @@ dialog_replace_get_dialog (void)
 	dialog->match_case_checkbutton = glade_xml_get_widget (gui, "match_case_checkbutton");
 	dialog->wrap_around_checkbutton =  glade_xml_get_widget (gui, "wrap_around_checkbutton");
 	dialog->entire_word_checkbutton = glade_xml_get_widget (gui, "entire_word_checkbutton");
+	dialog->search_backwards_checkbutton = glade_xml_get_widget (gui, "search_backwards_checkbutton");
 
 	if (!content				||
 	    !dialog->search_entry 		||
@@ -309,10 +308,12 @@ dialog_replace_get_dialog (void)
 	    !replace_with_label 		||
 	    !dialog->match_case_checkbutton 	||
 	    !dialog->entire_word_checkbutton 	||
-	    !dialog->wrap_around_checkbutton)
+	    !dialog->wrap_around_checkbutton 	||
+	    !dialog->search_backwards_checkbutton)
 	{
-		g_print
-		    ("Could not find the required widgets inside replace.glade2.\n");
+		gedit_warning (window,
+			       MISSING_WIDGETS,
+			       GEDIT_GLADEDIR "replace.glade2");
 		return NULL;
 	}
 
@@ -343,8 +344,6 @@ dialog_replace_get_dialog (void)
 
 	g_object_unref (G_OBJECT (gui));
 
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-	
 	return dialog;
 }
 
@@ -377,26 +376,28 @@ dialog_find_get_dialog (void)
 	
 	gedit_debug (DEBUG_SEARCH, "");
 
+	window = GTK_WINDOW (gedit_get_active_window ());
+
 	if (dialog != NULL)
 	{
-		gdk_window_show (dialog->dialog->window);
-		gdk_window_raise (dialog->dialog->window);
+		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
+					      GTK_WINDOW (window));
+		gtk_window_present (GTK_WINDOW (dialog->dialog));
 		gtk_widget_grab_focus (dialog->dialog);
+
 		return dialog;
 	}
 
 	gui = glade_xml_new (GEDIT_GLADEDIR "replace.glade2",
 			     "replace_dialog_content", NULL);
-
-	if (!gui) {
-		g_warning
-		    ("Could not find replace.glade2, reinstall gedit.\n");
+	if (!gui)
+	{
+		gedit_warning (window,
+			       MISSING_FILE,
+			       GEDIT_GLADEDIR "replace.glade2");
 		return NULL;
 	}
 
-	window = GTK_WINDOW (bonobo_mdi_get_active_window
-			     (BONOBO_MDI (gedit_mdi)));
-
 	dialog = g_new0 (GeditDialogFind, 1);
 
 	dialog->dialog = gtk_dialog_new_with_buttons (_("Find"),
@@ -410,6 +411,9 @@ dialog_find_get_dialog (void)
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
+	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->dialog), FALSE);
+
 	content = glade_xml_get_widget (gui, "replace_dialog_content");
 
 	dialog->search_entry       = glade_xml_get_widget (gui, "search_for_text_entry");
@@ -421,10 +425,12 @@ dialog_find_get_dialog (void)
 	dialog->match_case_checkbutton = glade_xml_get_widget (gui, "match_case_checkbutton");
 	dialog->wrap_around_checkbutton =  glade_xml_get_widget (gui, "wrap_around_checkbutton");
 	dialog->entire_word_checkbutton = glade_xml_get_widget (gui, "entire_word_checkbutton");
+	dialog->search_backwards_checkbutton = glade_xml_get_widget (gui, "search_backwards_checkbutton");
 
 	table                      = glade_xml_get_widget (gui, "table");
 
 	if (!content				||
+	    !table				||
 	    !dialog->search_entry 		||
 	    !dialog->search_entry_list 		||
 	    !replace_entry			||
@@ -432,13 +438,14 @@ dialog_find_get_dialog (void)
 	    !dialog->match_case_checkbutton 	||
 	    !dialog->entire_word_checkbutton 	||
 	    !dialog->wrap_around_checkbutton	||
-	    !table) 
+	    !dialog->search_backwards_checkbutton)
 	{
-		g_print
-		    ("Could not find the required widgets inside replace.glade2.\n");
+		gedit_warning (window,
+			       MISSING_WIDGETS,
+			       GEDIT_GLADEDIR "replace.glade2");
 		return NULL;
 	}
-	
+
 	gtk_widget_hide (replace_with_label);
 	gtk_widget_hide (replace_entry);
 
@@ -464,8 +471,6 @@ dialog_find_get_dialog (void)
 
 	g_object_unref (G_OBJECT (gui));
 
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-	
 	return dialog;
 }
 
@@ -551,7 +556,6 @@ insert_text_handler (GtkEditable *editable, const gchar *text, gint length, gint
 	g_free (escaped_text);
 }
 
-
 void
 gedit_dialog_find (void)
 {
@@ -564,26 +568,20 @@ gedit_dialog_find (void)
 	gboolean was_wrap_around;
 	gboolean was_entire_word;
 	gboolean was_case_sensitive;
+	gboolean was_search_backwards;
 	gpointer data;
 	
 	gedit_debug (DEBUG_SEARCH, "");
 
 	dialog = dialog_find_get_dialog ();
-	if (dialog == NULL) {
-		g_warning ("Cannot create the Find dialog");
+	if (!dialog)
 		return;
-	}
-
-	gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
-				      GTK_WINDOW
-				      (bonobo_mdi_get_active_window
-				       (BONOBO_MDI (gedit_mdi))));
 
 	if (GTK_WIDGET_VISIBLE (dialog->dialog))
 		return;
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-			GEDIT_RESPONSE_FIND, FALSE);
+					   GEDIT_RESPONSE_FIND, FALSE);
 
 	active_child = GEDIT_MDI_CHILD (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_child != NULL);
@@ -612,6 +610,9 @@ gedit_dialog_find (void)
 		}
 	}
 
+	if (!was_search_backwards_id)
+		was_search_backwards_id = g_quark_from_static_string ("GeditWasSearchBackwards");
+
 	if (!was_wrap_around_id)
 		was_wrap_around_id = gedit_was_wrap_around_quark ();
 
@@ -627,6 +628,12 @@ gedit_dialog_find (void)
 	else
 		was_wrap_around = GPOINTER_TO_BOOLEAN (data);
 
+	data = g_object_get_qdata (G_OBJECT (doc), was_search_backwards_id);
+	if (data == NULL)
+		was_search_backwards = FALSE;
+	else
+		was_search_backwards = GPOINTER_TO_BOOLEAN (data);
+
 	data = g_object_get_qdata (G_OBJECT (doc), was_entire_word_id);
 	if (data == NULL)
 		was_entire_word = FALSE;
@@ -645,11 +652,12 @@ gedit_dialog_find (void)
 				      was_entire_word);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton),
 				      was_wrap_around);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->search_backwards_checkbutton),
+				      was_search_backwards);
 
 	gtk_widget_grab_focus (dialog->search_entry);
 	
-	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
-		gtk_widget_show (dialog->dialog);
+	gtk_widget_show (dialog->dialog);
 }
 
 void
@@ -665,20 +673,14 @@ gedit_dialog_replace (void)
 	gboolean was_wrap_around;
 	gboolean was_entire_word;
 	gboolean was_case_sensitive;
+	gboolean was_search_backwards;
 	gpointer data;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
 	dialog = dialog_replace_get_dialog ();
-	if (dialog == NULL) {
-		g_warning ("Cannot create the Replace dialog");
+	if (!dialog)
 		return;
-	}
-
-	gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
-				      GTK_WINDOW
-				      (bonobo_mdi_get_active_window
-				       (BONOBO_MDI (gedit_mdi))));
 
 	if (GTK_WIDGET_VISIBLE (dialog->dialog))
 		return;
@@ -723,6 +725,10 @@ gedit_dialog_replace (void)
 		gtk_entry_set_text (GTK_ENTRY (dialog->replace_entry), last_replace_text);
 		g_free (last_replace_text);	
 	}
+
+	if (!was_search_backwards_id)
+		was_search_backwards_id = g_quark_from_static_string ("GeditWasSearchBackwards");
+
 	if (!was_wrap_around_id)
 		was_wrap_around_id = gedit_was_wrap_around_quark ();
 
@@ -732,6 +738,12 @@ gedit_dialog_replace (void)
 	if (!was_case_sensitive_id)
 		was_case_sensitive_id = g_quark_from_static_string ("GeditWasCaseSensitive");
 
+	data = g_object_get_qdata (G_OBJECT (doc), was_search_backwards_id);
+	if (data == NULL)
+		was_search_backwards = FALSE;
+	else
+		was_search_backwards = GPOINTER_TO_BOOLEAN (data);
+
 	data = g_object_get_qdata (G_OBJECT (doc), was_wrap_around_id);
 	if (data == NULL)
 		was_wrap_around = TRUE;
@@ -756,11 +768,12 @@ gedit_dialog_replace (void)
 				      was_entire_word);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton),
 				      was_wrap_around);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->search_backwards_checkbutton),
+				      was_search_backwards);
 
 	gtk_widget_grab_focus (dialog->search_entry);
-	
-	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
-		gtk_widget_show (dialog->dialog);
+
+	gtk_widget_show (dialog->dialog);
 }
 
 static void
@@ -770,10 +783,13 @@ find_dlg_find_button_pressed (GeditDialogFind *dialog)
 	GeditView* active_view;
 	GeditDocument *doc;
 	const gchar* search_string = NULL;
+	gboolean found;
+
 	gboolean case_sensitive;
 	gboolean entire_word;
 	gboolean wrap_around;
-	gboolean found;
+	gboolean search_backwards;
+	gint flags = 0;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
@@ -797,20 +813,33 @@ find_dlg_find_button_pressed (GeditDialogFind *dialog)
 
 	gnome_entry_prepend_history (GNOME_ENTRY (dialog->search_entry_list), TRUE, search_string);
 		
+	/* retrieve search settings from the dialog */
 	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
-
 	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
-	
 	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
+	search_backwards = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->search_backwards_checkbutton));
 
-	g_object_set_qdata (G_OBJECT (doc), was_wrap_around_id, GBOOLEAN_TO_POINTER (wrap_around ));
+	/* setup quarks for next invocation */
+	g_object_set_qdata (G_OBJECT (doc), was_search_backwards_id, GBOOLEAN_TO_POINTER (search_backwards));
+	g_object_set_qdata (G_OBJECT (doc), was_wrap_around_id, GBOOLEAN_TO_POINTER (wrap_around));
 	g_object_set_qdata (G_OBJECT (doc), was_entire_word_id, GBOOLEAN_TO_POINTER (entire_word));
 	g_object_set_qdata (G_OBJECT (doc), was_case_sensitive_id, GBOOLEAN_TO_POINTER (case_sensitive));
 
-	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+	/* setup search parameter bitfield */
+	GEDIT_SEARCH_SET_FROM_CURSOR (flags, TRUE);
+	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, case_sensitive);
+	GEDIT_SEARCH_SET_BACKWARDS (flags, search_backwards);
+	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
 
+	/* run search */
+	found = gedit_document_find (doc, search_string, flags);
+
+	/* if we're able to wrap, don't use the cursor position */
 	if (!found && wrap_around)
-		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+	{
+		GEDIT_SEARCH_SET_FROM_CURSOR (flags, FALSE);
+		found = gedit_document_find (doc, search_string, flags);
+	}
 
 	if (found)
 		gedit_view_scroll_to_cursor (active_view);
@@ -830,10 +859,13 @@ replace_dlg_find_button_pressed (GeditDialogReplace *dialog)
 	GeditView* active_view;
 	GeditDocument *doc;
 	const gchar* search_string = NULL;
+	gboolean found;
+
 	gboolean case_sensitive;
 	gboolean entire_word;
 	gboolean wrap_around;
-	gboolean found;
+	gboolean search_backwards;
+	gint flags = 0;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
@@ -857,20 +889,33 @@ replace_dlg_find_button_pressed (GeditDialogReplace *dialog)
 	
 	gnome_entry_prepend_history (GNOME_ENTRY (dialog->search_entry_list), TRUE, search_string);
 		
+	/* retrieve search settings from the dialog */
 	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
-
 	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
-
 	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
+	search_backwards = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->search_backwards_checkbutton));
 
+	/* setup quarks for next invocation */
+	g_object_set_qdata (G_OBJECT (doc), was_search_backwards_id, GBOOLEAN_TO_POINTER (search_backwards));
 	g_object_set_qdata (G_OBJECT (doc), was_wrap_around_id, GBOOLEAN_TO_POINTER (wrap_around));
 	g_object_set_qdata (G_OBJECT (doc), was_entire_word_id, GBOOLEAN_TO_POINTER (entire_word));
 	g_object_set_qdata (G_OBJECT (doc), was_case_sensitive_id, GBOOLEAN_TO_POINTER (case_sensitive));
 
-	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+	/* setup search parameter bitfield */
+	GEDIT_SEARCH_SET_FROM_CURSOR (flags, TRUE);
+	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, case_sensitive);
+	GEDIT_SEARCH_SET_BACKWARDS (flags, search_backwards);
+	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
 
+	/* run search */
+	found = gedit_document_find (doc, search_string, flags);
+
+	/* if we're able to wrap, don't use the cursor position */
 	if (!found && wrap_around)
-		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+	{
+		GEDIT_SEARCH_SET_FROM_CURSOR (flags, FALSE);
+		found = gedit_document_find (doc, search_string, flags);
+	}
 
 	if (found)
 		gedit_view_scroll_to_cursor (active_view);
@@ -893,11 +938,14 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	const gchar* replace_string = NULL;
 	gchar* selected_text = NULL;
 	gchar *converted_search_string = NULL;
+	gint start, end;
+	gboolean found;
+
 	gboolean case_sensitive;
 	gboolean entire_word;
-	gint start, end;
 	gboolean wrap_around;
-	gboolean found;
+	gboolean search_backwards;
+	gint flags = 0;
 
 	gedit_debug (DEBUG_SEARCH, "");
 
@@ -941,19 +989,19 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	gedit_debug (DEBUG_SEARCH, "Sel text: %s", selected_text ? selected_text : "NULL");
 	gedit_debug (DEBUG_SEARCH, "Search string: %s", search_string ? search_string : "NULL");
 
+	/* retrieve search settings from the dialog */
 	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
-
 	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
-
 	wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->wrap_around_checkbutton));
+	search_backwards = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->search_backwards_checkbutton));
 
 	converted_search_string = gedit_utils_convert_search_text (search_string);
 
 	if ((selected_text == NULL) ||
-	    (case_sensitive && (strcmp (selected_text, converted_search_string)
-	     !=0)) || (!case_sensitive && !g_utf8_caselessnmatch (selected_text, search_string, 
-						       strlen (selected_text), 
-						       strlen (search_string)) != 0))
+	    (case_sensitive && (strcmp (selected_text, converted_search_string) != 0)) || 
+	    (!case_sensitive && !g_utf8_caselessnmatch (selected_text, search_string, 
+						        strlen (selected_text), 
+						        strlen (search_string)) != 0))
 	{
 		gedit_debug (DEBUG_SEARCH, "selected_text (%s) != search_string (%s)", 
 			     selected_text ? selected_text : "NULL",
@@ -979,24 +1027,38 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 
 	gedit_debug (DEBUG_SEARCH, "Replaced");
 
-	/* go ahead and find the next one */
-	found = gedit_document_find (doc, search_string, TRUE, case_sensitive, entire_word);
+	/* setup search parameter bitfield */
+	GEDIT_SEARCH_SET_FROM_CURSOR (flags, TRUE);
+	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, case_sensitive);
+	GEDIT_SEARCH_SET_BACKWARDS (flags, search_backwards);
+	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
 
+	/* run search */
+	found = gedit_document_find (doc, search_string, flags);
+
+	gedit_debug (DEBUG_SEARCH, "Found 1: %s", found ? "TRUE" : "FALSE");
+
+	/* if we're able to wrap, don't use the cursor position */
 	if (!found && wrap_around)
-		found = gedit_document_find (doc, search_string, FALSE, case_sensitive, entire_word);
+	{
+		GEDIT_SEARCH_SET_FROM_CURSOR (flags, FALSE);
+		found = gedit_document_find (doc, search_string, flags);
+
+		gedit_debug (DEBUG_SEARCH, "Found 2: %s", found ? "TRUE" : "FALSE");
+	}
 
 	if (found)
 		gedit_view_scroll_to_cursor (active_view);
 	
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog->dialog), 
-							GEDIT_RESPONSE_REPLACE, found);	
+					   GEDIT_RESPONSE_REPLACE, 
+					   found);	
 
 	update_menu_items_sensitivity ();
 
 	gedit_debug (DEBUG_SEARCH, "END");
 }
 
-
 static void
 replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 {
@@ -1005,11 +1067,13 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 	GeditDocument *doc;
 	const gchar* search_string = NULL;
 	const gchar* replace_string = NULL;
-	gboolean case_sensitive;
-	gboolean entire_word;
 	gint replaced_items;
 	GtkWidget *message_dlg;
 	
+	gboolean case_sensitive;
+	gboolean entire_word;
+	gint flags = 0;
+	
  	gedit_debug (DEBUG_SEARCH, "");
 
 	if (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)) == NULL)
@@ -1040,12 +1104,15 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 					     TRUE, 
 					     replace_string);
 
+	/* retrieve search settings from the dialog */
 	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->match_case_checkbutton));
-
 	entire_word = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->entire_word_checkbutton));
 
-	replaced_items = gedit_document_replace_all (doc, search_string, replace_string, 
-						     case_sensitive, entire_word);
+	/* setup search parameter bitfield */
+	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, case_sensitive);
+	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, entire_word);
+
+	replaced_items = gedit_document_replace_all (doc, search_string, replace_string, flags);
 
 	update_menu_items_sensitivity ();
 
@@ -1065,7 +1132,9 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_MESSAGE_INFO,
 			GTK_BUTTONS_OK,
-			_("Found and replaced %d occurrences."), replaced_items);
+			ngettext("Found and replaced %d occurrence.",
+				 "Found and replaced %d occurrences.",
+				 replaced_items), replaced_items);
 	}
 	
 	gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
@@ -1075,4 +1144,3 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 	gtk_dialog_run (GTK_DIALOG (message_dlg));
   	gtk_widget_destroy (message_dlg);
 }
-
