@@ -121,7 +121,8 @@ typedef enum
 
   GEDIT_ENCODING_LAST,
 
-  GEDIT_ENCODING_UTF_8
+  GEDIT_ENCODING_UTF_8,
+  GEDIT_ENCODING_UNKNOWN
   
 } GeditEncodingIndex;
 
@@ -130,6 +131,13 @@ static GeditEncoding utf8_encoding =
 	  "UTF-8", 
 	  N_("Unicode") 
 	};
+
+static GeditEncoding unknown_encoding = 
+	{ GEDIT_ENCODING_UNKNOWN,
+	  NULL, 
+	  NULL 
+	};
+
 	
 static GeditEncoding encodings [] = {
 
@@ -262,15 +270,13 @@ static GeditEncoding encodings [] = {
 static void
 gedit_encoding_lazy_init (void)
 {
-
 	static gboolean initialized = FALSE;
 	gint i;
-	
+	const gchar *locale_charset;
+
 	if (initialized)
 		return;
 
-	g_return_if_fail (G_N_ELEMENTS (encodings) == GEDIT_ENCODING_LAST);
-  
 	i = 0;
 	while (i < GEDIT_ENCODING_LAST)
 	{
@@ -279,11 +285,17 @@ gedit_encoding_lazy_init (void)
 		/* Translate the names */
 		encodings[i].name = _(encodings[i].name);
       
+		
 		++i;
     	}
 
 	utf8_encoding.name = _(utf8_encoding.name);
 
+	if (g_get_charset (&locale_charset) == FALSE)
+	{
+		unknown_encoding.charset = g_strdup (locale_charset);
+	}
+
 	initialized = TRUE;
 }
 
@@ -292,6 +304,8 @@ gedit_encoding_get_from_charset (const gchar *charset)
 {
 	gint i;
 
+	g_return_val_if_fail (charset != NULL, NULL);
+
 	gedit_encoding_lazy_init ();
 
 	if (charset == NULL)
@@ -300,6 +314,12 @@ gedit_encoding_get_from_charset (const gchar *charset)
 	if (g_ascii_strcasecmp (charset, "UTF-8") == 0)
 		return gedit_encoding_get_utf8 ();
 
+	if (unknown_encoding.charset != NULL)
+	{
+		if (g_ascii_strcasecmp (charset, unknown_encoding.charset) == 0)
+			return &unknown_encoding;
+	}
+
 	i = 0; 
 	while (i < GEDIT_ENCODING_LAST)
 	{
@@ -308,7 +328,7 @@ gedit_encoding_get_from_charset (const gchar *charset)
       
 		++i;
 	}
- 
+
 	return NULL;
 }
 
@@ -348,19 +368,25 @@ gedit_encoding_get_current (void)
 
 	if (g_get_charset (&locale_charset) == FALSE) 
 	{
-		if (locale_charset != NULL)
-			locale_encoding = gedit_encoding_get_from_charset (locale_charset);
-	
+		g_return_val_if_fail (locale_charset != NULL, &utf8_encoding);
+		
+		locale_encoding = gedit_encoding_get_from_charset (locale_charset);
 	}
 	else
 	{
 		locale_encoding = &utf8_encoding;
 	}
 	
+	if (locale_encoding == NULL)
+	{
+		locale_encoding = &unknown_encoding;
+	}
+
+	g_return_val_if_fail (locale_encoding != NULL, NULL);
+
 	initialized = TRUE;
 
 	return locale_encoding;
-
 }
 	
 
@@ -368,12 +394,19 @@ gchar *
 gedit_encoding_to_string (const GeditEncoding* enc)
 {
 	g_return_val_if_fail (enc != NULL, NULL);
-	g_return_val_if_fail (enc->name != NULL, NULL);
 	g_return_val_if_fail (enc->charset != NULL, NULL);
 
 	gedit_encoding_lazy_init ();
 
-    	return g_strdup_printf ("%s (%s)", enc->name, enc->charset);
+	if (enc->name != NULL)
+	    	return g_strdup_printf ("%s (%s)", enc->name, enc->charset);
+	else
+	{
+		if (g_ascii_strcasecmp (enc->charset, "ANSI_X3.4-1968") == 0)
+			return g_strdup_printf ("US-ASCII (%s)", enc->charset);
+		else
+			return g_strdup (enc->charset);
+	}
 }
 
 const gchar *
