@@ -229,29 +229,9 @@ dialog_response_handler (GtkDialog *dlg, gint res_id,  ShellOutputDialog *dialog
 			gtk_widget_destroy (dialog->dialog);
 
 			break;
-
 		}
 }
 
-static void
-display_error_dialog (GtkWindow *parent)
-{
-	GtkWidget *err_dialog;
-	
-	err_dialog = gtk_message_dialog_new (
-			parent,
-			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-		   	GTK_MESSAGE_ERROR,
-		   	GTK_BUTTONS_OK,
-			_("An error occured while running the selected command."));
-			
-	gtk_dialog_set_default_response (GTK_DIALOG (err_dialog), GTK_RESPONSE_OK);
-	gtk_window_set_resizable (GTK_WINDOW (err_dialog), FALSE);
-
-	gtk_dialog_run (GTK_DIALOG (err_dialog));
-	gtk_widget_destroy (err_dialog);
-}
-
 static ShellOutputDialog*
 get_dialog (void)
 {
@@ -284,10 +264,11 @@ get_dialog (void)
 
 	gui = glade_xml_new (GEDIT_GLADEDIR "shell_output.glade2",
 			     "shell_output_dialog_content", NULL);
-
-	if (!gui) {
-		g_warning
-		    ("Could not find shell_output.glade2, reinstall gedit.\n");
+	if (!gui)
+	{
+		gedit_warning (window,
+			       MISSING_FILE,	
+			       GEDIT_GLADEDIR "shell_output.glade2");
 		return NULL;
 	}
 
@@ -298,7 +279,6 @@ get_dialog (void)
 
 	dialog->toplevel_window = window;
 
-
 	dialog->dialog = gtk_dialog_new_with_buttons (_("Run Command"),
 						      window,
 						      GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -308,6 +288,9 @@ get_dialog (void)
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
+	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->dialog), FALSE);
+
 	/* Add the run button */
 	dialog->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog->dialog), 
 					GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
@@ -317,27 +300,29 @@ get_dialog (void)
 	dialog->run_button =  gedit_dialog_add_button (GTK_DIALOG (dialog->dialog), 
 				 _("_Run"), GTK_STOCK_EXECUTE, GTK_RESPONSE_OK);
 
-	content			= glade_xml_get_widget (gui, "shell_output_dialog_content");
-
-	g_return_val_if_fail (content != NULL, NULL);
-
+	content	= glade_xml_get_widget (gui, "shell_output_dialog_content");
 	dialog->command    	= glade_xml_get_widget (gui, "command_entry");
 	dialog->command_list   	= glade_xml_get_widget (gui, "command_entry_list");
-
 	dialog->directory  	= glade_xml_get_widget (gui, "directory_entry");
 	dialog->directory_fileentry = glade_xml_get_widget (gui, "directory_fileentry");
 	dialog->capture_output  = glade_xml_get_widget (gui, "capture_ouput_checkbutton");
-
 	dialog->command_label  	= glade_xml_get_widget (gui, "command_label");
 	dialog->directory_label	= glade_xml_get_widget (gui, "directory_label");
 
-	g_return_val_if_fail (dialog->command != NULL, NULL);
-	g_return_val_if_fail (dialog->command_label != NULL, NULL);
-	g_return_val_if_fail (dialog->command_list != NULL, NULL);
-	g_return_val_if_fail (dialog->directory != NULL, NULL);
-	g_return_val_if_fail (dialog->directory_fileentry != NULL, NULL);
-	g_return_val_if_fail (dialog->directory_label != NULL, NULL);
-	g_return_val_if_fail (dialog->capture_output != NULL, NULL);
+	if (!content ||
+	    !dialog->command             ||
+	    !dialog->command_label       ||
+	    !dialog->command_list        ||
+	    !dialog->directory           ||
+	    !dialog->directory_fileentry ||
+	    !dialog->directory_label     ||
+	    !dialog->capture_output)
+	{
+		gedit_warning (window,
+			       MISSING_WIDGETS,
+			       GEDIT_GLADEDIR "docinfo.glade2");
+		return NULL;
+	}
 
 	gtk_entry_set_text (GTK_ENTRY (dialog->directory), current_directory);
 
@@ -348,21 +333,18 @@ get_dialog (void)
 					 GTK_RESPONSE_OK);
 
 	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
-			   G_CALLBACK (dialog_destroyed), &dialog);
+			  G_CALLBACK (dialog_destroyed), &dialog);
 
 	g_signal_connect (G_OBJECT (dialog->dialog), "response",
-			   G_CALLBACK (dialog_response_handler), dialog);
+			  G_CALLBACK (dialog_response_handler), dialog);
 
 	g_object_unref (gui);
 
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-
 	gtk_widget_grab_focus (dialog->command);
 
 	gtk_widget_show (dialog->dialog);
 
 	return dialog;
-
 }
 
 static void
@@ -373,13 +355,10 @@ run_command_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 	gedit_debug (DEBUG_PLUGINS, "");
 
 	dialog = get_dialog ();
-	if (dialog == NULL) {
-		g_warning ("Could not create the Shell Output dialog");
+	if (!dialog)
 		return;
-	}
 }
 
-
 static gboolean
 handle_command_output (GIOChannel *ioc, GIOCondition condition, gpointer data) 
 {
@@ -693,23 +672,9 @@ run_command_real (ShellOutputDialog *dialog)
 
 	if (command_string == NULL || (strlen (command_string) == 0))
 	{
-		GtkWidget *err_dialog;
-		
-		err_dialog = gtk_message_dialog_new (
-				GTK_WINDOW (dialog->dialog),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-			   	GTK_MESSAGE_ERROR,
-			   	GTK_BUTTONS_OK,
+		gedit_warning (GTK_WINDOW (dialog->dialog),
 				_("The shell command entry is empty.\n\n"
 				  "Please, insert a valid shell command."));
-			
-		gtk_dialog_set_default_response (GTK_DIALOG (err_dialog), GTK_RESPONSE_OK);
-
-		gtk_window_set_resizable (GTK_WINDOW (err_dialog), FALSE);
-
-		gtk_dialog_run (GTK_DIALOG (err_dialog));
-  		gtk_widget_destroy (err_dialog);
-
 		return FALSE;
 	}
 
@@ -726,23 +691,9 @@ run_command_real (ShellOutputDialog *dialog)
                            NULL, &argv,
                            NULL))
 	{
-		GtkWidget *err_dialog;
-		
-		err_dialog = gtk_message_dialog_new (
-				GTK_WINDOW (dialog->dialog),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-			   	GTK_MESSAGE_ERROR,
-			   	GTK_BUTTONS_OK,
+		gedit_warning (GTK_WINDOW (dialog->dialog),
 				_("Error parsing the shell command.\n\n"
 				  "Please, insert a valid shell command."));
-			
-		gtk_dialog_set_default_response (GTK_DIALOG (err_dialog), GTK_RESPONSE_OK);
-
-		gtk_window_set_resizable (GTK_WINDOW (err_dialog), FALSE);
-
-		gtk_dialog_run (GTK_DIALOG (err_dialog));
-  		gtk_widget_destroy (err_dialog);
-
 		g_free (unescaped_command_string);
 		
 		return FALSE;
@@ -886,10 +837,10 @@ run_command_real (ShellOutputDialog *dialog)
 	{
 		running = NOT_RUNNING;
 
-		display_error_dialog (GTK_WINDOW (dialog->dialog));
-		
-		g_free (unescaped_command_string);
+		gedit_warning (GTK_WINDOW (dialog->dialog),
+			       _("An error occured while running the selected command."));
 
+		g_free (unescaped_command_string);
 		gtk_widget_destroy (dialog->dialog);
 
 		return FALSE;
